//+------------------------------------------------------------------+
//|                                                   BB Color.mq5 |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+

#property copyright   "2009, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property description "Bollinger Bands Color"

//---- include directive
#include <MovingAverages.mqh>

//---- Property directive
#property indicator_chart_window
#property indicator_buffers 7
#property indicator_plots   3

#property indicator_type1   DRAW_COLOR_LINE
#property indicator_type2   DRAW_COLOR_LINE
#property indicator_type3   DRAW_COLOR_LINE

#property indicator_color1  clrSilver, clrLime, clrRed
#property indicator_color2  clrSilver, clrLime, clrRed
#property indicator_color3  clrSilver, clrLime, clrRed

#property indicator_style1  STYLE_DOT
#property indicator_style2  STYLE_SOLID
#property indicator_style3  STYLE_SOLID

#property indicator_width1  1
#property indicator_width2  2
#property indicator_width3  2

#property indicator_label1  "Bands middle"
#property indicator_label2  "Bands upper"
#property indicator_label3  "Bands lower"

//---- input parameters
input int     InpBandsPeriod = 20;             // Period
input int     InpBandsShift = 0;               // Shift
input double  InpBandsDeviations = 2.0;        // Deviation

//---- global variables
int           ExtBandsPeriod,ExtBandsShift;
double        ExtBandsDeviations;
int           ExtPlotBegin=0;

//---- data indicator buffer
double        ExtBandMidBuffer[];              // buffer 0
double        ExtBandUppBuffer[];              // buffer 1
double        ExtBandLowBuffer[];              // buffer 2
double        ExtStdDevBuffer[];               // buffer 3

//---- colors indicator buffer
double        ExtColorBandMidBuffer[];         // color buffer 4
double        ExtColorBandUppBuffer[];         // color buffer 5
double        ExtColorBandLowBuffer[];         // color buffer 6

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

void OnInit()
  {
   //---- check for input values
   if(InpBandsPeriod<2)
     {
      ExtBandsPeriod=18;
      printf("Incorrect value for input variable InpBandsPeriod=%d. Indicator will use value=%d for calculations.",InpBandsPeriod,ExtBandsPeriod);
     }
   else ExtBandsPeriod=InpBandsPeriod;
   if(InpBandsShift<0)
     {
      ExtBandsShift=0;
      printf("Incorrect value for input variable InpBandsShift=%d. Indicator will use value=%d for calculations.",InpBandsShift,ExtBandsShift);
     }
   else
      ExtBandsShift=InpBandsShift;
   if(InpBandsDeviations==0.0)
     {
      ExtBandsDeviations=2.0;
      printf("Incorrect value for input variable InpBandsDeviations=%f. Indicator will use value=%f for calculations.",InpBandsDeviations,ExtBandsDeviations);
     }
   else ExtBandsDeviations=InpBandsDeviations;
   
   //---- assign the data array with indicator's buffer
   SetIndexBuffer(0,ExtBandMidBuffer, INDICATOR_DATA);
   SetIndexBuffer(1,ExtBandUppBuffer, INDICATOR_DATA);
   SetIndexBuffer(2,ExtBandLowBuffer, INDICATOR_DATA);
   SetIndexBuffer(3,ExtStdDevBuffer,INDICATOR_CALCULATIONS);
   
   //---- Specify the number of color indexes, used in the graphic plot
   SetIndexBuffer(4,ExtColorBandMidBuffer,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(5,ExtColorBandUppBuffer,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(6,ExtColorBandLowBuffer,INDICATOR_COLOR_INDEX);
   
   //---- Set index labels
   PlotIndexSetString(0,PLOT_LABEL,"Bands("+string(ExtBandsPeriod)+") Middle");
   PlotIndexSetString(1,PLOT_LABEL,"Bands("+string(ExtBandsPeriod)+") Upper");
   PlotIndexSetString(2,PLOT_LABEL,"Bands("+string(ExtBandsPeriod)+") Lower");

   //---- Set Indicator name
   IndicatorSetString(INDICATOR_SHORTNAME,"Bollinger Bands Bicolor");

   //---- Indexes draw begin settings
   ExtPlotBegin=ExtBandsPeriod-1;
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,ExtBandsPeriod);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,ExtBandsPeriod);
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,ExtBandsPeriod);
 
   //---- Indexes shift settings
   PlotIndexSetInteger(0,PLOT_SHIFT,ExtBandsShift);
   PlotIndexSetInteger(1,PLOT_SHIFT,ExtBandsShift);
   PlotIndexSetInteger(2,PLOT_SHIFT,ExtBandsShift);
   
   //---- Number of digits of indicator value
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
   
   //---- nInit done
  }
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int OnCalculate(const int     rates_total,
                const int     prev_calculated,
                const int     begin,
                const double  &price[])

  {
   //---- Variables
   int pos;
   
   //---- indexes draw begin settings, when we've recieved previous begin
   if(ExtPlotBegin!=ExtBandsPeriod+begin)
     {
      ExtPlotBegin=ExtBandsPeriod+begin;
      PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,ExtPlotBegin);
      PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,ExtPlotBegin);
      PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,ExtPlotBegin);
     }

   //---- sheck for bars count
   if(rates_total<ExtPlotBegin)
      return(0);

   //---- starting calculation
   if(prev_calculated>1) pos=prev_calculated-1;
      else pos=0;

   //---- main loop cycle
   for(int i=pos; i<rates_total && !IsStopped(); i++)
   {
      //---- middle line
      ExtBandMidBuffer[i]=SimpleMA(i,ExtBandsPeriod,price);
      
      //---- calculate and write down StdDev
      ExtStdDevBuffer[i]=StdDev_Func(i,price,ExtBandMidBuffer,ExtBandsPeriod);
      
      //---- upper line
      ExtBandUppBuffer[i]=ExtBandMidBuffer[i]+ExtBandsDeviations*ExtStdDevBuffer[i];
      
      //---- lower line
      ExtBandLowBuffer[i]=ExtBandMidBuffer[i]-ExtBandsDeviations*ExtStdDevBuffer[i];
   }

   //---- loop color envelopes of Bollinger
   for(int i=pos+1; i<rates_total && !IsStopped(); i++)
   {
      //---- middle line
      //ExtColorBandMidBuffer[i]=0;
      if (ExtBandMidBuffer[i] > ExtBandMidBuffer[i-1]) // &&(ExtBandMidBuffer[i+1] > ExtBandMidBuffer[i+2]))
         {
            ExtColorBandMidBuffer[i]=1;
         }
      else
         {
            ExtColorBandMidBuffer[i]=2;
         }

//      //---- upper line
//      //ExtColorBandUppBuffer[i]=0;
      if (ExtBandUppBuffer[i] > ExtBandUppBuffer[i-1]) // &&(ExtBandUppBuffer[i+1] > ExtBandUppBuffer[i+2]))
         {
            ExtColorBandUppBuffer[i]=1;
         }
      else
         {
            ExtColorBandUppBuffer[i]=2;
         }
//      
//      //---- lower line
//      //ExtColorBandLowBuffer[i]=0;
      if (ExtBandLowBuffer[i] > ExtBandLowBuffer[i-1]) // &&(ExtBandLowBuffer[i+1] > ExtBandLowBuffer[i+2]))
         {
            ExtColorBandLowBuffer[i]=1;
         }
      else
         {
            ExtColorBandLowBuffer[i]=2;
         }
   }

   //---- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }
  
//+------------------------------------------------------------------+
//| Calculate Standard Deviation                                     |
//+------------------------------------------------------------------+

double StdDev_Func(int position,const double &price[],const double &MAprice[],int period)
  {
   //---- Variables
   double StdDev_dTmp=0.0;
   
   //---- Check for position
   if(position<period) return(StdDev_dTmp);

   //---- Calcualte StdDev
   for(int i=0;i<period;i++) StdDev_dTmp+=MathPow(price[position-i]-MAprice[position],2);
   StdDev_dTmp=MathSqrt(StdDev_dTmp/period);

   //---- Return calculated value
   return(StdDev_dTmp);
  }

//+------------------------------------------------------------------+